/* This beacon code first connects to the internet, connects to an mqtt broker, then opens radio receiving to detect if transmitters are near by
 * Integration and kitbashing by Christopher Nase
 *
 * See documentation at https://nRF24.github.io/RF24
 * See License information at root directory of this library
 * Author: Brendan Doherty 2bndy5
 */

#include "WiFi.h"
#include <PubSubClient.h>
#include <Wire.h>
#include <SPI.h>
#include "printf.h"
#include "RF24.h"

//WiFi Login
const char* ssid = ""; //insert wifi name
const char* password = ""; //insert password

const char* mqtt_server = "test.mosquitto.org"; //The mqtt server used for this project

WiFiClient espClient;
PubSubClient client(espClient);

RF24 radio(4, 5);  // using pin 4 for the CE pin, and pin 5 for the CSN pin

int lastID = 0; //The following two are used for calculating the over all "signal strength"
int runningSum = 0;

String message = ""; //What to send
String beaconID = "1"; //Which beacon is this

struct PayloadStruct {
  unsigned long nodeID;
  unsigned long payloadID;
};

PayloadStruct payload;
void setup(){
  Serial.begin(115200);
  radio.begin();

  Serial.println("Radio Starting up...");
  if(!radio.begin()){
    Serial.println("Radio not good...");
    delay(1000);
  }
  Serial.print("Connecting to ");
  Serial.println(ssid);
  
  WiFi.begin(ssid, password);

  while(WiFi.status() != WL_CONNECTED){
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.print("WiFi Connected!");

  client.setServer(mqtt_server, 1883);

  radio.setPALevel(RF24_PA_LOW);
  radio.setPayloadSize(sizeof(payload));
  radio.openReadingPipe(0, 0xB3B4B5B605LL);
  radio.startListening();
}

//Sends either a 1 or a 0 depending on if there is an incoming transmission
int recData(){
  uint8_t pipe;
  if(radio.available(&pipe)){
    uint8_t bytes = radio.getPayloadSize();
    radio.read(&payload,bytes);
    return 1;
  }
  delay(200);
  return 0;
}

void loop(){
  if(recData()){
    if(payload.payloadID > lastID){ //If the the radio transmission is still ongoing
      runningSum = runningSum + payload.payloadID;
    }
    else{ //Formats the time and such to publish a string containing all the formatted data ['day hour',cat,beaconID,signalStrength]
      unsigned long currentMillis;
      unsigned long seconds;
      unsigned long minutes;
      unsigned long hours;
      unsigned long days;

      currentMillis = millis();
      seconds = currentMillis / 1000;
      minutes = seconds / 60;
      hours = minutes / 60;
      days = hours / 24;
      currentMillis %= 1000;
      seconds %= 60;
      minutes %= 60;
      hours %= 24;

      float signalStrength = map(runningSum, 0, 4950,0,10); //Takes the number of transmissions and uses it to map a signal strength from 0 to 10
      
      message = "";
      message += "'";
      message += days+1;
      message += " ";
      message += (hours+(minutes/24));
      message += "',";
      message += payload.nodeID;
      message += ",";
      message += beaconID;
      message += ",";
      message += signalStrength;

      char Buf[40]; //Convert the message into a character array
      message.toCharArray(Buf, 40);     

      client.connect("ESP32Client"); //Connect to the mqtt server
      client.publish("esp32test/", Buf); //Publish it
      Serial.println("Yahooo!");
      delay(500);
      runningSum = 0; //reset to prep for next transmission
    }
  }
}


